-- Visualize scalar data written in COMSOL export format using pdelib2

processargs(arg)

-- split line into tokens 
function split (line)
   local tbn={}   
   string.gsub(line,"([%w_\.\+\-]+)",function(s) table.insert(tbn,s) end)
   return tbn
end

comsolname=arg.files[1]

if not comsolname or arg.switches.help then

   print[[

	 pdelib2 comsolview file [-grid] [-view2] [title="title"]
	 
	 Visualize scalar data written in COMSOL export format. 

   BUGS/TODO: Calculation of domain boundary would allow better
	 function and grid visualization
   ]]
   os.exit()
end

-- open comsol data file
comsol=io.open(comsolname)


coord_mode=1
cell_mode=2
data_mode=3


-- scan comsol data file
for rawline in comsol:lines() do
   line=split(rawline)

-- encounter %
   if string.sub(rawline,1,1)=="%" then

-- encounter % Coordinates
      if line[1]=="Coordinates" then
	 print("Coordinates...")
	 mode=coord_mode
	 icoord=1
	 -- space dimension is read in from first coordinates line

-- encounter % Elements
      elseif line[1]=="Elements" then
	 -- coordinate read finished
	 print("ncoord:",icoord-1)


	 print("Elements...")
	 mode=elem_mode
	 cells=ia2Create(0,dim+1)
	 icell=1

-- encounter % Data
      elseif line[1]=="Data" then 
	 -- element read finished
	 print("ncells:",icell-1)

	 -- create regions array
	 regions=ia1Create(icell-1)
	 for i=1,icell-1 do regions[i]=1 end

	 -- create dummy boundary consisting of one
	 -- facet. This needs to be changed: we shall
	 -- calculate the domain boudary from element
	 -- data.
	 facets=ia2Create(1,dim)
	 facetregions=ia1Create(1)
	 for j=1,dim do
	    facets[1][j]=cells[1][j]
	 end
	 facetregions[1]=1

	 -- create grid
	 grid=gCreate(coord,cells,regions,facets,facetregions)
	 gPrintStatus(grid)

	 -- create grid function holding data
	 space=gsCreateFV(grid,gNode)
	 data=da1CreateS(space)
	 print("Data")
	 mode=data_mode
	 idata=1
      end

-- encounter Data lines

-- line contains coordinates
   elseif mode==coord_mode then
      -- detect dimension from first coordinate line
      if icoord==1 then
	 dim=table.getn(line)
	 print("Dim=",dim)
	 coord=da2Create(0,dim)
      end
      for i=1,dim do da2Insert(coord,icoord,i,tonumber(line[i])) end
      icoord=icoord+1

-- line contains element node indexes
   elseif mode==elem_mode then
      for i=1,dim+1 do ia2Insert(cells,icell,i,tonumber(line[i])) end
      icell=icell+1

-- line contains data
   elseif mode==data_mode then
      data[idata]=tonumber(line[1])
      idata=idata+1
   end
end

-- data read finished
print("ndata:",idata-1)

-- close comsol file
comsol:close()

-- call viewer
viewer=glvCreate(0,0,800,500,comsolname)
if arg.values.title then glvSetTitle(viewer,arg.values.title) end

if arg.switches.grid and arg.switches.view2 then
   gView2(viewer,grid)
elseif arg.switches.grid then
   gView(viewer,grid)
elseif arg.switches.view2 then
   aView2(viewer,data)
else
   aView(viewer,data)
end
glvWait()

