need("mbase")

--- 
--- \file wrap_gnuplot.lua 
--- \brief Lua Wrapper for gnuplot class
---

------------------------------------------------------------
--- \brief
--- \luaonly Wrapper constructor based on description table.
--- \see gnuplotCreate()
--+ GnuPlot  *GnuPlotNew(GnuplotClass *self,table desc)

function gnuplotNew(self,...)
   local desc=arg[1]	
   local title,x,y,w,h
   title="pdelib2"
   x=0
   y=0
   w=400
   h=300
   if desc then 
    if desc.title then title=mydesc.title end
    if desc.x then x=mydesc.x end
    if desc.y then y=mydesc.y end
    if desc.w then w=mydesc.w end
    if desc.h then h=mydesc.h end
   end
   local g=mgraphic.gnuplotCreate(x,y,w,(3*w)/4,title)
   swigext_extend(g,"plot")
   swigext_extend(g,"cmdf")
   swigext_extend(g,"cmd")
   swigext_extend(g,"equation")
   g.plot=wrap_gnuplotPlot
   g.cmdf=wrap_gnuplotCmdF
   g.cmd=wrap_gnuplotCmd
   g.equation=wrap_gnuplotPlotEquation
   return g	
end


-- wrap bound C function
function wrap_gnuplotCmdF(gp,fmt,...)
  mgraphic.gnuplotCmd(gp,string.format(fmt,unpack(arg)))	 
end

function wrap_gnuplotCmd(gp,cmd)
  mgraphic.gnuplotCmd(gp,cmd)	 
end

-- wrap bound C function
-- it works as gnuplotPlot, gnuplotPlotA1, gnuplotPlotA2
-- and p:plot{a1,a2;col={1,3}, axes=... title=...} 
function wrap_gnuplotPlot(gp,...)
local a1,a2,axes,title=""
local myarg

	if type(arg[1])=="table" and type(arg[1][1])~="number" then myarg=arg[1] else myarg=arg end
	if myarg.title then title=myarg.title end

	if istype(myarg[1],"DArray1") or istype(myarg[1],"table") then 
--	   a1=Array(myarg[1])
	   a1=DArray1(myarg[1])
	   a2=a1
	elseif istype(myarg[1],"DArray2") then
	   mgraphic.gnuplotPlotA2(gp, myarg[1],myarg[2],myarg[3],title)
	   return
	else
	   error("gnuplot: first data argument missing or not Array")
	   return 
        end

	if istype(myarg[2],"DArray1") or istype(myarg[2],"table") then
--         a2=Array(myarg[2])
           a2=DArray1(myarg[2])
           title=myarg[3] 
        else
	   error("gnuplot: second data argument missing, or not (Array or number)")
          return
        end
        if not title then title="" end
	swigext_basecall(gp,"plot",a1,a2,title)
end


function wrap_gnuplotPlotEquation(gp,...)
local eq,title
local myarg
if istype(arg[1],"table") then myarg=arg[1] else myarg=arg end
  eq=myarg[1]
  title=myarg[2]
  title=myarg.title
  if not title then title=eq end
  mgraphic.gnuplotPlotEquation(gp,eq,title)	
end


GnuPlot=gnuplotNew


--[[
GnuPlot.cmd=gnuplotCmd
GnuPlot.xlabel=gnuplotSetXLabel
GnuPlot.axes=gnuplotSetAxes
GnuPlot.style=gnuplotSetStyle
GnuPlot.ylabel=gnuplotSetYLabel
GnuPlot.reset=gnuplotReset
GnuPlot.replot=gnuplotRePlot
GnuPlot.immediate=gnuplotImmediate
GnuPlot.equation=gnuplotPlotEquation
GnuPlot.plot=gnuplotPlot
GnuPlot.new=gnuplotNew
GnuPlot.wait=gnuplotWaitUser

getmetatable(GnuPlot)["__call"]=callnew
]]








------------------------------------------------------------
--- \brief
--- \luaonly Constructor based on description table.
--+ gnuplot_create(table desc)

function gnuplot_create(win)
  
 if type(win) ~= "table" then return end
 
 local x=10
 local y=10
 local w=100
 local h=100
 
 if  win.x ~= nil
 and win.y ~= nil
 and win.w ~= nil
 and win.h ~= nil then
  x=win.x
  y=win.y       
  w=win.w
  h=win.h
 end
 
 local title="gnuplot_window"
 
 if win.title ~= nil then
  title=win.title
 end
 
 return gnuplotCreate(x,y,w,h,title)
end


------------------------------------------------------------
--- \brief
--- \luaonly Setting visualization properties based on description table.
--+ gnuplot_visu(GnuPlot* gp, table desc)

function gnuplot_visu(gp, visu)
  
 if type(visu) ~= "table" then return end
 
 if type(visu.xrange) == "table" and table.getn(visu.xrange) >= 2 then
  gnuplotCmd(gp, string.format("set xrange [%.5f:%.5f]", visu.xrange[1],visu.xrange[2]))
 end
 
 if type(visu.yrange) == "table" and table.getn(visu.yrange) >= 2 then
  gnuplotCmd(gp, string.format("set yrange [%.5f:%.5f]", visu.yrange[1],visu.yrange[2]))
 end
 
 if type(visu.logscale) == "table" then
  if table.getn(visu.logscale) >= 2 then
   gnuplotCmd(gp,"set logscale "..visu.logscale[1].." "..visu.logscale[2])
  elseif table.getn(visu.logscale) == 1 then
   gnuplotCmd(gp,"set logscale "..visu.logscale[1])
  end
 end
 
 if visu.title ~= nil then gnuplotCmd(gp,"set title '"..visu.title.."'") end
 
 if visu.xlabel ~= nil then gnuplotSetXLabel(gp,visu.xlabel) end
 if visu.ylabel ~= nil then gnuplotSetYLabel(gp,visu.ylabel) end
 
 if visu.xtics ~= nil then gnuplotCmd(gp,"set xtics "..visu.xtics) end
 if visu.ytics ~= nil then gnuplotCmd(gp,"set ytics "..visu.ytics) end
 if visu.pointsize ~= nil then gnuplotCmd(gp,"set pointsize "..visu.pointsize) end
 
 if visu.style ~= nil then gnuplotSetStyle(gp,visu.style) end
  
end


------------------------------------------------------------
--- \brief
--- \luaonly Setting visualization properties and reset based on description table.
--+ gnuplot_reset_visu(GnuPlot* gp, table desc)
function gnuplot_reset_visu(gp, visu)
 gnuplotReset(gp)
 gnuplot_visu(gp,visu)
end





